/*
** $Id: strings.c,v 1.2 2002/09/24 22:05:13 pief Exp $
**
** MetalServe common stuff
** Copyright (c) 2002 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** String functions
**
** $Log: strings.c,v $
** Revision 1.2  2002/09/24 22:05:13  pief
** For some reason I checked in ancient versions, so here are the current
** ones
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#include "base.h"
#include "strings.h"

/*
** string = lower_string(string)
**
** Converts the supplied string to lower case.
**
** Returns "string".
*/
char *lower_string(char *string)
{
	char *p;

	for (p = string; *p != '\0'; p++) *p = tolower(*p);

	return string;
}

/*
** trimmed_line = trim_line(line)
**
** Skips heading and removes trailing whitespace (incl. tabs) and trailing
** end-of-line chars (CR, LF, CR/LF).
**
** Returns a pointer to the first non-whitespace character.
*/
char *trim_line(char *line)
{
	char *p;

	/* Skip heading whitespace and tabs */
	line += strspn(line, " \t");

	/* Find last character before terminating null byte */
	p = strchr(line, '\0') - 1;

	/* Remove whitespace, tabs, CR and LF at end of line */
	while ( (*p == '\n') || (*p == '\r') || (*p == ' ') || (*p == '\t') )
	{
		*p = '\0';
		p--;
	}

	return line;
}

/*
** unescaped_string = unescape(string)
**
** Replaces all escape sequenced with the original characters. Escape
** sequences start with a backslash ('\'), followed by the original
** character.
**
** Returns a copy of "string", which must be free()'d manually.
*/
char *unescape(char *string)
{
        char *new_string;
        char *src, *dest;
        BOOL escaped = FALSE;
                 
        /* Allocate memory for new string (size <= old string) */
        new_string = malloc(strlen(string)+1);
        if (!new_string) return NULL;
  
        for (src = string, dest = new_string; *src; src++)
        {
                if (*src == '\\')
                {
                        if (escaped)
                        {
                                *dest = '\\';
                                dest++;
                                escaped = FALSE;
                        }
                        else escaped = TRUE;
                }
                else if (*src == '"')
                {
                        if (escaped)
                        {
                                *dest = '"';
                                dest++;
                                escaped = FALSE;
                        }
                }
                else
                {
                        *dest = *src;
                        dest++;
                }
        }
                                
        /* Make sure to terminate the copy */
        *dest = '\0';

	return new_string;	
}

#if 0
/*
** split_args(line, first, rest)
**
** Extracts the first string from a series of space-seperated strings.
** "line" is the string to be parsed, "first" and "rest" are pointers
** to variables to be set.
**
** If spaces are used within the argument, the entire argument must be
** put in double quotes (eg. "arg with spaces"). Alternatively, the
** single space may be escaped with a preceeding backslash. If the
** entire argument is quoted, a single-standing quote within the argument
** must be escaped with a preceeding backslash as well.
**
** While heading and trailing quotes are removed, escaped chars are left
** "as-is". Use unescape() if necessary.
**
** "first" will be set to the extracted first argument.
** "rest" will be set to the remaining string of arguments or NULL.
*/
void split_args(char *line, char **first, char **rest)
{
	char prevchar = '\0';
	int quoted;

	/* Maybe no argument was specified at all... */
	if (!line) return;

	/* Skip heading whitespace (multiple spaces between args or
	   space remaining from previous quoted argument) */
	*first = line;
	while ( (**first == ' ') || (**first == '\t') ) *first = *first + 1;

	/* Check for quoted argument */
	if (**first == '"')
	{
		quoted = TRUE;
		*first = *first + 1;
	}
	else quoted = FALSE;

	*rest = *first;

	/* Look for next argument */
	for (*rest = *first; (*rest) && (**rest != '\0'); *rest = *rest + 1)
	{
		/* Treat whitespace as arg seperator unless quoted or
		   escaped */
		if (
		     (**rest == ' ') && (quoted == FALSE) && (prevchar != '\\')
		   )
		{
			**rest = '\0';
			*rest  = *rest + 1;

			return;
		}

		/* In quoted mode, the closing quote ends both quoting
		   mode and the argument */
		if (
		     (quoted == TRUE) && (**rest == '"') && (prevchar != '\\')
		   )
		{
			**rest = '\0';
			quoted = FALSE;
		}

		/* Remember this character, maybe it's a backslash */
		prevchar = **rest;
	}

	/* No more arguments were found */
	*rest = NULL;
}
#endif

